<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:plm="http://www.plmxml.org/Schemas/PLMXMLSchema"
	xmlns:cim="http://www.coolemall.eu/DEBBComponent" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xmlns:fun="http://whatever">

	<!-- Importing xslt for translating debb components -->
	<xsl:import href="DEBBComponentTranslator.xsl" />

	<!-- Information from which level the translator should start moving information to templates -->
	<xsl:param name="templateStartLevel" select="templateStartLevel" />
	
	<xsl:param name="plmxmlFileName" select="plmxmlFileName" />
		
	<!-- Specification of the output document -->
	<xsl:output method="xml" version="1.0" encoding="UTF-8"
		indent="yes" />

	<!-- Root of the PLM XML document -->
	<xsl:variable name="rootDoc" select="/" />



	<xsl:template match="/">
		<xsl:element name="environment">
			<xsl:attribute name="xsi:noNamespaceSchemaLocation">../DCwormsResSchema.xsd</xsl:attribute>
			
			<xsl:element name="resources">
				<xsl:attribute name="mode">CoolEmAll</xsl:attribute>
			
				<!-- Iterate the document, create hierarchy and "links" to templates -->
				<xsl:apply-templates select="//plm:InstanceGraph" mode="createHierarchy"/>
								
				<!-- Create main scheduler -->
				<xsl:element name="scheduler"></xsl:element>
			</xsl:element>
			
			<!-- Create templates (if needed) -->
			<xsl:if test="fun:containsTemplates() = true()">
				<xsl:element name="templates">
					<!-- Iterate the document and createTemplates -->
					<xsl:apply-templates select="//plm:InstanceGraph" mode="createTemplates"/>
				</xsl:element>			
			</xsl:if>
			
		</xsl:element>
	</xsl:template>

	<xsl:template match="plm:InstanceGraph" mode="createHierarchy">
		<!-- Finding root DEBB -->
		<xsl:variable name="rootRef" select="@rootRefs" />
		<xsl:apply-templates select="plm:ProductInstance[@id=$rootRef]" mode="createHierarchy"/>
	</xsl:template>

	<xsl:template match="plm:InstanceGraph" mode="createTemplates">
     <xsl:apply-templates select="//plm:ProductRevisionView[plm:UserData/plm:UserValue/@title='DEBBLevel' and plm:UserData/plm:UserValue/@value=$templateStartLevel]" mode="createTemplates"/>

		
	</xsl:template>

	<xsl:template match="plm:ProductInstance" mode="createHierarchy">
		<xsl:variable name="partRef" select="@partRef" />
		<!-- Get rid of # at the beginning -->
		<xsl:variable name="productInstanceId" select="translate($partRef, '#', '')" />

		<xsl:variable name="productInstanceType" select="../plm:ProductRevisionView[@id=$productInstanceId]/plm:UserData/plm:UserValue[@title='DEBBLevel']/@value"/>
		<xsl:if test="not ($productInstanceType = 'Sensor' or $productInstanceType ='Heatsink' or $productInstanceType ='Inlet' or $productInstanceType = 'Outlet' or $productInstanceType = 'CoolingDevice' or $productInstanceType ='Powersupply' or $productInstanceType ='Wall')">
			<!-- We have to get rid of sensors, heatsinks, inlets, outlets, which make only problems here -->
						
			<xsl:element name="computingResource">
				<xsl:attribute name="name"><xsl:value-of select="@name" /></xsl:attribute>

				<!-- Find corresponding ProductRevisionView and let it translate itself -->
				<xsl:apply-templates select="//plm:ProductRevisionView[@id=$productInstanceId]" mode="createHierarchy"/>

				<!-- Add hostnames, labels, locations, sequences, etc. -->
				<xsl:variable name="debbHostname"
					select="plm:UserData/plm:UserValue[@title='hostname']/@value" />
				<xsl:if test="$debbHostname != ''">
					<xsl:element name="parameter">
						<xsl:attribute name="name">hostname</xsl:attribute>
						<xsl:element name="value">
							<xsl:value-of select="$debbHostname" />
						</xsl:element>
					</xsl:element>
				</xsl:if>

			</xsl:element>
		</xsl:if>
		<xsl:if test="$productInstanceType = 'CoolingDevice' or $productInstanceType ='Heatsink' or $productInstanceType ='Inlet' or $productInstanceType = 'Outlet'">
			<xsl:element name="device">
				<xsl:attribute name="name">
					<xsl:value-of select="@name" />
				</xsl:attribute>

				<!-- Find corresponding ProductRevisionView and let it translate itself -->
				<xsl:apply-templates select="//plm:ProductRevisionView[@id=$productInstanceId]" mode="createDevice"/>
					
			</xsl:element>
		</xsl:if>		
	</xsl:template>
	

	<xsl:template match="plm:ProductRevisionView" mode="createHierarchy">

		<!-- instanceRefs contain space separated ids of nested DEBBs -->
		<xsl:variable name="tokenizedText" select="tokenize(@instanceRefs, ' ')" />
	
		<!-- DEBBLevel -->
		<xsl:variable name="debbLevel"
			select="plm:UserData/plm:UserValue[@title='DEBBLevel']/@value" />
		<!-- DEBBComponentId -->
		<xsl:variable name="debbComponentId"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentId']/@value" />

		<xsl:variable name="debbComponentFile"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentFile']/@value" />

					
		<xsl:attribute name="class">
			<xsl:value-of select="$debbLevel"/>
		</xsl:attribute>
		
		<xsl:choose>
			<xsl:when test="$debbLevel = $templateStartLevel">
				<!-- Change into template. The rest of the hierarchy will be in a corresponding template. -->
				<xsl:element name="templateId">
					<xsl:value-of select="$debbLevel"/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select="$debbComponentId"/>
				</xsl:element>		
			</xsl:when>
			<xsl:otherwise>
			
				<!-- Read information from DEBBComponent file -->	
				<xsl:choose>
					 <!-- DEBB Components file is defined  -->
					<xsl:when test="$debbComponentFile != ''">
						<xsl:variable name="nestedDoc" select="document($debbComponentFile)" />
						<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]" mode="component">
							<xsl:with-param name="instanceRefs" select="@instanceRefs" />		
						</xsl:apply-templates>
					</xsl:when>

				</xsl:choose>			
			
				<!-- Iterate children and ask them to translate themselves -->
				<xsl:for-each select="1 to count($tokenizedText)">
					<xsl:variable name="index" select="." />
					<xsl:apply-templates
						select="$rootDoc//plm:ProductInstance[@id=$tokenizedText[$index]]" mode="createHierarchy"/>
				</xsl:for-each>			
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<xsl:template match="plm:ProductRevisionView" mode="createDevice">
	
		<!-- DEBBLevel -->
		<xsl:variable name="debbLevel"
			select="plm:UserData/plm:UserValue[@title='DEBBLevel']/@value" />
		<!-- DEBBComponentId -->
		<xsl:variable name="debbComponentId"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentId']/@value" />

		<xsl:variable name="debbComponentFile"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentFile']/@value" />

					
		<xsl:attribute name="class">
			<xsl:value-of select="$debbLevel"/>
		</xsl:attribute>

		 <!-- DEBB Components file is defined  -->
		<xsl:if test="$debbComponentFile != ''">
			<xsl:variable name="nestedDoc" select="document($debbComponentFile)" />

			<xsl:if test="$debbLevel = 'CoolingDevice'">
				<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]" mode="coolingDevice">
					<xsl:with-param name="instanceRefs" select="@id" />		
				</xsl:apply-templates>
			</xsl:if>	
			<xsl:if test="$debbLevel = 'Inlet' or $debbLevel = 'Outlet'">
				<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]" mode="fan">
					<xsl:with-param name="instanceRefs" select="@id" />		
				</xsl:apply-templates>
			</xsl:if>
			<xsl:if test="$debbLevel = 'Heatsink'">
				<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]" mode="heatsink">
					<xsl:with-param name="instanceRefs" select="@id" />		
				</xsl:apply-templates>
			</xsl:if>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="plm:ProductRevisionView" mode="createTemplates">

		<!-- instanceRefs contain space separated ids of nested DEBBs -->
		<xsl:variable name="tokenizedText" select="tokenize(@instanceRefs, ' ')" />
	
		<xsl:variable name="debbLevel"
			select="plm:UserData/plm:UserValue[@title='DEBBLevel']/@value" />
			
		<xsl:variable name="debbComponentId"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentId']/@value" />

		<xsl:variable name="debbComponentFile"
			select="plm:UserData/plm:UserValue[@title='DEBBComponentFile']/@value" />

	
		<xsl:if test="$debbLevel">
			<xsl:element name="computingResourceTemplate">
				<xsl:attribute name="name">
					<xsl:value-of select="$debbLevel"/>
					<xsl:text>-</xsl:text>
					<xsl:value-of select="$debbComponentId"/>
				</xsl:attribute>
				<xsl:attribute name="class">
					<xsl:value-of select="$debbLevel"/>
				</xsl:attribute>
				
				<!-- Read information from DEBBComponent file -->	
				<xsl:choose>
					 <!-- DEBB Components file is defined  -->
					<xsl:when test="$debbComponentFile != ''">
						<xsl:variable name="nestedDoc" select="document($debbComponentFile)" />

						<xsl:apply-templates select="$nestedDoc//*[ComponentId=$debbComponentId]" mode="component"> 
							<xsl:with-param name="instanceRefs" select="@instanceRefs" />							
						</xsl:apply-templates>
					
					</xsl:when>

				</xsl:choose>
				
				<!-- Iterate children and ask them to translate themselves -->
				<xsl:for-each select="1 to count($tokenizedText)">
					<xsl:variable name="index" select="." />
					<xsl:apply-templates
						select="$rootDoc//plm:ProductInstance[@id=$tokenizedText[$index]]" mode="createHierarchy"/>
				</xsl:for-each>			
							
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="plm:ProductInstance" mode="updateHeatsinkDevice">

		<xsl:variable name="power-sensor"
			select="plm:UserData/plm:UserValue[@title='power-sensor']/@value" />
		<xsl:choose>
	    	<xsl:when test="$power-sensor != ''">
				<xsl:element name="parameter">
					<xsl:attribute name="name">power-sensor</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$power-sensor" />
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="parameter">
					<xsl:attribute name="name">power-sensor</xsl:attribute>
					<xsl:element name="value">power</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	
		<!-- 
		<xsl:variable name="temperature-sensor"
			select="plm:UserData/plm:UserValue[@title='temperature-sensor']/@value" />
		<xsl:choose>
			<xsl:when test="$temperature-sensor != ''">
				<xsl:element name="parameter">
					<xsl:attribute name="name">temperature-sensor</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$temperature-sensor" />
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="parameter">
					<xsl:attribute name="name">temperature-sensor</xsl:attribute>
					<xsl:element name="value">temperature</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
		-->
	</xsl:template>
	
	<xsl:template match="plm:ProductInstance" mode="updateCoolingDevice">

		<xsl:variable name="airflow_volume-sensor"
			select="plm:UserData/plm:UserValue[@title='airflow_volume-sensor']/@value" />
		<xsl:choose>
			<xsl:when test="$airflow_volume-sensor != ''">
				<xsl:element name="parameter">
					<xsl:attribute name="name">airflow_volume-sensor</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$airflow_volume-sensor" />
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="parameter">
					<xsl:attribute name="name">airflow_volume-sensor</xsl:attribute>
					<xsl:element name="value">airflow_volume</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
		
		<xsl:variable name="airflow_speed-sensor"
			select="plm:UserData/plm:UserValue[@title='airflow_speed-sensor']/@value" />
		<xsl:choose>	
			<xsl:when test="$airflow_speed-sensor != ''">
				<xsl:element name="parameter">
					<xsl:attribute name="name">airflow_speed-sensor</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$airflow_speed-sensor" />
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="parameter">
					<xsl:attribute name="name">airflow_speed-sensor</xsl:attribute>
					<xsl:element name="value">airflow_speed</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:variable name="temperature-sensor"
			select="plm:UserData/plm:UserValue[@title='temperature-sensor']/@value" />
		<xsl:choose>
			<xsl:when test="$temperature-sensor != ''">
				<xsl:element name="parameter">
					<xsl:attribute name="name">temperature-sensor</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$temperature-sensor" />
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="parameter">
					<xsl:attribute name="name">temperature-sensor</xsl:attribute>
					<xsl:element name="value">temperature</xsl:element>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:function name="fun:containsTemplates">
		<xsl:variable name="productRevisionViews" select="$rootDoc//plm:ProductRevisionView[plm:UserData/plm:UserValue/@title='DEBBLevel' and plm:UserData/plm:UserValue/@value=$templateStartLevel]"/>
		<xsl:variable name="numberOfTemplates" select="count($productRevisionViews)"/>

		<xsl:value-of select="$numberOfTemplates &gt; 0"/>
	</xsl:function>	
	
</xsl:stylesheet>

